#This Python script makes a request to generate a unique API Key that can be attached to a user's account.
#The script then displays the API Key as a QR code representation that can be scanned using a mobile app.
#The "refresh" function is included to download a list of available models and present them using omni.ui.
#Finally, the script includes functionality to download a model as a USDZ file and insert it into an Omniverse scene.
from functools import partial
from genericpath import exists
import os
import omni.ext
import omni.ui as ui
import omni.usd
import omni.kit.commands
from omni.kit.window.filepicker.style import get_style
from pxr import Gf
from pxr import UsdGeom
from .api import get_model_url, download_file, load_models, get_preview_url, downloadApikey, downloadQrCode


cache_dir = ""
config_dir = "settings/config.cfg"
qr_code_dir = "settings/qrCode.png"
apiKey = ""
notLinkedText = f"Not connected with MagiScan AI\r\n\r\nOpen App, Scan QR Code and\r\ntap refresh button"
check_internet_connection = "Check Internet Connection and tap Refresh button"
linkedText = "Plugin sucesfully connected to MagiScan AI App"
loadingText = f"Loading..."

def get_path_in_stage():
    stage = omni.usd.get_context().get_stage()

    for i in range(100):
        p = stage.GetPrimAtPath(f"/World/Magiscan_{i:>02}")
        if not p.IsValid():
            return f"/World/Magiscan_{i:>02}"
    else:
        assert False


def add_model_to_stage(url, path_in_stage):
    stage = omni.usd.get_context().get_stage()

    prim = UsdGeom.Xform.Define(stage, path_in_stage)
    prim.AddScaleOp().Set(Gf.Vec3f(1000.0, 1000.0, 1000.0))
    refs = prim.GetPrim().GetReferences()
    refs.AddReference(url)


class MagiscanExtension(omni.ext.IExt):
    status_label = None
    scrolling_frame = None

    def on_startup(self, ext_id):
        global cache_dir
        cache_dir = (
            omni.kit.app.get_app().get_extension_manager().get_extension_path(ext_id)
            + "/cache"
        )

        if not self.isInited(cache_dir):
            self.init(cache_dir)
        
        self.apiKey = self.get_token(cache_dir)
        qr_code_path = f"{cache_dir}/{qr_code_dir}"

        print("[argeneration.magiscan.ai] magiscan extension startup")

        self._window = ui.Window("MagiScan AI 3D Scanner App", width=600, height=350)
        with self._window.frame:
            with ui.VStack(height=10):
                with ui.CollapsableFrame("Settings"):
                    with ui.VStack():
                        # Token
                        with ui.HStack(height=24, spacing=2, style=get_style()):
                            self.qrImage= ui.Image(
                                qr_code_path,
                                height=100,
                                width=100,
                            )
                            ui.Spacer(width=10)
                            with ui.VStack():
                                self.status_label = ui.Label(
                                notLinkedText,
                                width=0)
                                ui.Spacer(height=4)
                                ui.Spacer()
                                ui.Spacer(height=4)
                                ui.Button(
                                    "Refresh",
                                    width=230,
                                    height=20,
                                    clicked_fn=self.refresh,
                                    style_type_name_override="Button")
                ui.Spacer(height=4)
                with ui.CollapsableFrame("Models") as ff:
                    self.frame = ui.Frame()
        self.refresh()
    
    def refresh(self):
        result = load_models(self.apiKey) 

        if result == "NotLinked":
            self.qrImage.visible = True
            self.status_label.visible = True
            self.status_label.text = notLinkedText
            if self.scrolling_frame is not None:
                self.scrolling_frame.visible = False
            return

        if result == "NetworkError":
            self.status_label.text = check_internet_connection   
            return
        
        self.qrImage.visible =  False
        self.status_label.text = linkedText

        with self.frame:
            self.scrolling_frame = ui.ScrollingFrame(
                name="models",
                height=200,
                horizontal_scrollbar_policy=ui.ScrollBarPolicy.SCROLLBAR_ALWAYS_ON,
                vertical_scrollbar_policy=ui.ScrollBarPolicy.SCROLLBAR_ALWAYS_OFF,
            )

            with self.scrolling_frame:
                with ui.HStack(width=100):
                    for model in result:
                        with ui.VStack():
                            modelId = model["id"]
                            modelName = model["name"]
                            preview_path = f"{cache_dir}/previews/{modelId}.jpg"
                            if not os.path.exists(preview_path):
                                url = get_preview_url(model["previewUrl"])
                                download_file(url,preview_path)
                            ui.Image(
                                        preview_path,
                                        height=120,
                                        width=120,
                                    )
                            with ui.HStack():
                                ui.Spacer(width=18)
                                ui.Label(
                                    modelName)
                                ui.Spacer()
                            ui.Button(
                                    "Add",
                                    clicked_fn=partial(self.add_model, modelId=modelId),
                                )


    def isInited(self, cachePath):
        config_path = f"{cachePath}/{config_dir}"
        qr_code_path = f"{cachePath}/{qr_code_dir}"
        return os.path.exists(config_path) & os.path.exists(qr_code_path)


    def init(self, cachePath):
        config_path = f"{cachePath}/{config_dir}"
        qr_code_path = f"{cachePath}/{qr_code_dir}"
        apiKey = downloadApikey(config_path)
        downloadQrCode(apiKey, qr_code_path)


    def get_token(self, cachePath):
        config_path = f"{cachePath}/{config_dir}"
        with open(config_path, 'r') as file:
            apiKey = file.read()
        return apiKey


    def add_model(self, modelId):
        model_path = os.path.abspath(
            f"{cache_dir}/models/{modelId}.usdz"
        )
        if not os.path.exists(model_path):
            model_url = get_model_url(modelId, self.apiKey)
            download_file(model_url, model_path)
        path_in_stage = get_path_in_stage()
        add_model_to_stage(model_path, path_in_stage)


    def on_shutdown(self):
        print("[argeneration.magiscan.ai] magiscan extension shutdown")